/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is Chad House and 4067479
 * Canada Inc. t/a CELTX.
 * 
 * Portions created by Chad House are Copyright (C) 2000-2004 Chad House,
 * parts created by Celtx are Copyright (C) 4067479 Canada Inc. All Rights
 * Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */

var gWindow = {
  inPrintPreview: false
};


var gController = {
  QueryInterface: function QueryInterface (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsISupportsWeakReference) ||
        iid.equals(Components.interfaces.nsIController) ||
        iid.equals(Components.interfaces.nsIObserver) ||
        iid.equals(Components.interfaces.nsIDOMEventListener) ||
        iid.equals(Components.interfaces.nsIClipboardDragDropHooks))
      return this;
    else
      throw Components.results.NS_NOINTERFACE;
  },


  controllers: [
    gTitleController,
    gScriptController,
    gReportController,
    gCardController,
    gPDFController
  ],


  activeController: gScriptController,


  commands: {
    "cmd-export-script": 1,
    "cmd-import-script": 1,
    "cmd-script-format": 1,
    "cmd-toggle-sidebar": 1
  },


  supportsCommand: function supportsCommand (cmd) {
    if (this.commands[cmd] == 1)
      return true;
    for (var i = 0; i < this.controllers.length; ++i) {
      if (this.controllers[i].supportsCommand(cmd))
        return true;
    }
    return false;
  },


  isCommandEnabled: function isCommandEnabled (cmd) {
    if (gWindow.inPrintPreview)
      return false;

    switch (cmd) {
      default:
        return this.activeController.supportsCommand(cmd) &&
               this.activeController.isCommandEnabled(cmd);
    }
  },


  doCommand: function doCommand (cmd) {
    switch (cmd) {
      default:
        this.activeController.doCommand(cmd);
    }
  },


  updateCommands: function updateCommands () {
    for (var cmd in this.commands)
      goUpdateCommand(cmd);
    this.activeController.updateCommands();
    top.gFrameLoader.updateMenuCommands();
  },


  get modified () {
    for (var i = 0; i < this.controllers.length; ++i) {
      if (this.controllers[i].modified)
        return true;
    }
    return false;
  },


  open: function open (project, docres) {
    this.project = project;
    this.docres = docres;

    for (var i = 0; i < this.controllers.length; ++i)
      this.controllers[i].open(project, docres);
  },


  close: function close () {
    for (var i = 0; i < this.controllers.length; ++i)
      this.controllers[i].close();
  },


  focus: function focus () {
    this.focused = true;
    this.activeController.focus();
    this.updateCommands();
  },


  blur: function blur () {
    this.focused = false;
    this.activeController.blur();
  },


  save: function save () {
    for (var i = 0; i < this.controllers.length; ++i) {
      try {
        this.controllers[i].save();
      }
      catch (ex) {
        var ctrlname = "gTitleController";
        switch (i) {
          case 1: ctrlname = "gScriptController"; break;
          case 2: ctrlname = "gReportController"; break;
          case 3: ctrlname = "gCardController"; break;
          case 4: ctrlname = "gPDFController"; break;
        }
        dump("*** save: Failed in " + ctrlname + "\n");
        throw ex;
      }
    }
  },


  onScriptLoad: function onScriptLoad () {
    for (var i = 0; i < this.controllers.length; ++i) {
      if ("onScriptLoad" in this.controllers[i]) {
        try {
          this.controllers[i].onScriptLoad();
        }
        catch (ex) {
          dump("*** onScriptLoad: " + ex + "\n");
        }
      }
    }
  }
};


// Since menuitems are imported into the top-level document, we can't just
// check the ones in the current document, we need to pass it up to the top.
function setMenuChecked (menuid, checked) {
  var menuitem = top.document.getElementById(menuid);
  if (! menuitem)
    return;
  if (checked)
    menuitem.setAttribute("checked", true);
  else
    menuitem.removeAttribute("checked");
}


function loaded () {
  gWindow = new Object;

  for (var i = 0; i < gController.controllers.length; ++i) {
    try {
      gController.controllers[i].loaded();
    }
    catch (ex) {
      dump("*** this.controllers[" + i + "].loaded: " + ex + "\n");
    }
  }

  window.controllers.appendController(gController);

  var rdfsvc = getRDFService();
  var rdfsrc = currentProfileDir();
  rdfsrc.append(Cx.PREFS_FILE);
  gWindow.deptds = rdfsvc.GetDataSourceBlocking(fileToFileURL(rdfsrc));
  gPanel.deptPopup.database.AddDataSource(gWindow.deptds);
  gPanel.deptPopup.builder.rebuild();
}


function getBrowser () {
  if ("browser" in gController.activeController)
    return gController.activeController.browser;
  return null;
}


function getController () {
  return gController;
}


function getMenuPopup () {
  return document.getElementById("script-popup");
}


function setOutlineView (view) {
  dump("--- gController.setOutlineView\n");
  gController.outlineView = view;
}


function viewTabSelected (event) {
  var cardname = event.target.getAttribute("value");
  if (! cardname)
    return;
  var card = document.getElementById(cardname);
  if (! card)
    return;
  var deck = document.getElementById("cardsdeck");
  var oldcardname = deck.selectedPanel.id;
  if (oldcardname == cardname)
    return;
  gController.activeController.blur();

  switch (cardname) {
    case "scriptcard":
      gController.activeController = gScriptController; break;
    case "indexcard":
      gController.activeController = gCardController;   break;
    case "titlecard":
      gController.activeController = gTitleController;  break;
    case "reportcard":
      gController.activeController = gReportController; break;
    case "pdfcard":
      gController.activeController = gPDFController;    break;
  }
  deck.selectedPanel = card;
  gController.activeController.focus();
  gController.updateCommands();
}
