/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is Chad House and 4067479
 * Canada Inc. t/a CELTX.
 * 
 * Portions created by Chad House are Copyright (C) 2000-2004 Chad House,
 * parts created by Celtx are Copyright (C) 4067479 Canada Inc. All Rights
 * Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */

var gWindow;
var gSelectedScene;


function loaded () {
  gWindow = new Object;

  gWindow.deck          = document.getElementById("scriptnavdeck");
  gWindow.sceneTree     = document.getElementById("scene-tree");
  gWindow.scratchToggle = document.getElementById("toggle-scratchpad-button");
  gWindow.reportTree    = document.getElementById("report-tree");
}


function showSceneNav () {
  gWindow.deck.selectedIndex = 0;
}


function showReportNav () {
  gWindow.deck.selectedIndex = 1;
}


function showScratchpadButton () {
  gWindow.scratchToggle.hidden = false;
}


function hideScratchpadButton () {
  gWindow.scratchToggle.hidden = true;
}


function selectReportIndex (index) {
  gWindow.reportTree.view.selection.select(0);
}


function getSelectedReportIndex () {
  return gWindow.reportTree.view.selection.currentIndex;
}


function getSelectedReportName () {
  var idx = getSelectedReportIndex();
  var cols = gWindow.reportTree.columns;
  return gWindow.reportTree.view.getCellText(idx, cols.getFirstColumn());
}


function open (project, docres) {
  gWindow.ds = project.ds;
  gWindow.sceneTree.database.AddDataSource(gWindow.ds);
  gWindow.sceneTree.ref = docres.Value;
  gWindow.sceneTreeView = new SceneTreeView(gWindow.sceneTree);
  gWindow.sceneTree.builderView.addObserver(gWindow.sceneTreeView);
}


function close () {
  gWindow.sceneTree.builderView.removeObserver(gWindow.sceneTreeView);
  gWindow.sceneTree.database.RemoveDataSource(gWindow.ds);
}


// The controller from the primary script window
function setDelegate (delegate) {
  gWindow.delegate = delegate;
}


function setScratchpadState (open) {
  if (open)
    gWindow.scratchToggle.setAttribute("checked", "true");
  else
    gWindow.scratchToggle.removeAttribute("checked");
}


function onSceneSelect (event) {
  updateTreeCommands();
  if (gWindow.delegate)
    gWindow.delegate.onSceneSelect(event);
}


function updateTreeCommands () {
  goUpdateCommand("cmd-treeitem-down");
  goUpdateCommand("cmd-treeitem-up");
  goUpdateCommand("cmd-treeitem-recycle");
  goUpdateCommand("cmd-treeitem-delete");
}


function getSelectedSceneID () {
  var view = gWindow.sceneTree.view;
  var index = view.selection.currentIndex;
  if (index < 0)
    return null;
  var res = view.getResourceAtIndex(index);
  if (! res)
    return null;
  var ds = gWindow.sceneTree.database;
  var rdfsvc = getRDFService();
  var sceneidarc = rdfsvc.GetResource(Cx.NS_CX + "sceneid");
  return getRDFString(ds, res, sceneidarc);
}


function getSelectedScene () {
  var view = gWindow.sceneTree.view;
  var index = view.selection.currentIndex;
  if (index < 0)
    return null;
  return view.getResourceAtIndex(index);
}


function selectScene (sceneres) {
  var view = gWindow.sceneTree.view;
  var rdfsvc = getRDFService();
  var index = view.getIndexOfResource(sceneres);
  if (index >= 0) {
    view.selection.select(index);
    gWindow.sceneTree.treeBoxObject.ensureRowIsVisible(index);
  }
}


function selectSceneAtIndex (index) {
  gWindow.sceneTree.view.selection.select(index);
}


function SceneTreeView (tree) {
  this.tree = tree;
  this.dragsvc = Components.classes["@mozilla.org/widget/dragservice;1"]
    .getService(Components.interfaces.nsIDragService);
}


SceneTreeView.prototype = {
  QueryInterface: function (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsIXULTreeBuilderObserver))
      return true;
    throw Components.results.NS_NOINTERFACE;
  },


  DROP_BEFORE: Components.interfaces.nsIXULTreeBuilderObserver.DROP_BEFORE,
  DROP_ON: Components.interfaces.nsIXULTreeBuilderObserver.DROP_ON,
  DROP_AFTER: Components.interfaces.nsIXULTreeBuilderObserver.DROP_AFTER,


  sceneFlavour: "x-celtx/x-sceneid",


  canDrop: function (index, orientation) {
    if (! gWindow.delegate)
      return false;

    if (orientation == this.DROP_ON)
      return false;

    var dragSession = this.dragsvc.getCurrentSession();
    if (! dragSession)
      return false;

    if (dragSession.numDropItems != 1 ||
        ! dragSession.isDataFlavorSupported(this.sceneFlavour))
      return false;

    return true;
  },


  onCycleCell: function (row, colid) {},
  onCycleHeader: function (colid, elt) {},


  onDrop: function (row, orientation) {
    if (! gWindow.delegate) {
      dump("*** onDrop: no delegate\n");
      return;
    }

    var dragSession = this.dragsvc.getCurrentSession();
    if (! dragSession) {
      dump("*** onDrop: no dragSession\n");
      return;
    }

    if (dragSession.numDropItems != 1 ||
        ! dragSession.isDataFlavorSupported(this.sceneFlavour)) {
      dump("*** onDrop: bad numDropItems or flavour not supported\n");
      return;
    }

    try {
    var trans = Components.classes["@mozilla.org/widget/transferable;1"]
      .createInstance(Components.interfaces.nsITransferable);
    trans.addDataFlavor(this.sceneFlavour);
    dragSession.getData(trans, 0);
    var data = {};
    var len = {};
    trans.getTransferData(this.sceneFlavour, data, len);
    data = data.value.QueryInterface(Components.interfaces.nsISupportsString);
    if (! data) {
      dump("*** onDrop: no data\n");
      return;
    }
    data = data.data.substring(0, len.value);

    if (orientation == this.DROP_AFTER)
      ++row;

    // scene numbers are one-based, not zero-based
    // data = data.replace(/^.*\//, "");
    gWindow.delegate.moveSceneToIndex(data, row + 1);
    }
    catch (ex) {
      dump("*** onDrop: " + ex + "\n");
    }
  },


  onPerformAction: function (action)                {},
  onPerformActionOnCell: function (action, row, colid) {},
  onPerformActionOnRow: function (action, row)      {},
  onSelectionChanged: function ()                   {},
  onToggleOpenState: function (index) {},


  dragGesture: function dragGesture (event) {
    if (event.originalTarget.localName == "treechildren" && gWindow.delegate)
      nsDragAndDrop.startDrag(event, this);
  },


  onDragStart: function onDragStart (aEvent, aXferData, aDragAction) {
    try {
      var selection = this.tree.view.selection;
      if (selection.count != 1) {
        dump("*** onDragStart: multi-select, aborting\n");
        return;
      }
      var index = selection.currentIndex;
      var sceneres = this.tree.view.getResourceAtIndex(index);
      var ILit = Components.interfaces.nsIRDFLiteral;
      var ds = this.tree.database;
      var sceneidarc = getRDFService().GetResource(Cx.NS_CX + "sceneid");
      var sceneid = ds.GetTarget(sceneres, sceneidarc, true);
      if (! sceneid || ! (sceneid instanceof ILit)) {
        dump("*** onDragStart: No scene id for " + sceneres.Value + "\n");
        return;
      }
      sceneid = sceneid.QueryInterface(ILit).Value;
      var data = new TransferData();
      data.addDataForFlavour(this.sceneFlavour, sceneid);
      aXferData.data = data;
    }
    catch (ex) {
      dump("*** onDragStart: " + ex + "\n");
    }
  }
};
